Package: iterors
Type: Package
Title: Fast, Compact Iterators and Tools
Version: 1.0.1
Maintainer: Peter Meilstrup <peter.meilstrup@gmail.com>
Authors@R: c(person("Peter", "Meilstrup",
                    role=c("cre", "aut", "cph"), email="peter.meilstrup@gmail.com"),
             person("Folashade", "Daniel", role="aut"),
             person("Revolution Analytics", role=c("aut", "cph")),
             person("Steve", "Weston", role=c("aut", "cph")),
             person("John A.", "Ramey", role=c("aut", "cph")),
             person("Kayla", "Schaefer", role="aut"),
             person("Hadley", "Wickham", role="aut"))
Description: A fresh take on iterators in R. Designed to be cross-compatible with the 'iterators' package, but using the 'nextOr' method will offer better performance as well as more compact code. With batteries included: includes a collection of iterator constructors and combinators ported and refined from the 'iterators', 'itertools', and 'itertools2' packages.
Depends: R (>= 4.1)
Imports: rlang
Encoding: UTF-8
Suggests: reticulate, combinat, testthat (>= 3.0.0), foreach, iterators
        (>= 1.0.7), rmarkdown, knitr
URL: https://github.com/crowding/iterors,
        https://crowding.github.io/iterors/
License: GPL (>= 3)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: false
NeedsCompilation: no
Packaged: 2026-02-28 07:48:33 UTC; peter
Author: Peter Meilstrup [cre, aut, cph],
  Folashade Daniel [aut],
  Revolution Analytics [aut, cph],
  Steve Weston [aut, cph],
  John A. Ramey [aut, cph],
  Kayla Schaefer [aut],
  Hadley Wickham [aut]
Repository: CRAN
Date/Publication: 2026-02-28 08:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 09:12:05 UTC; unix
