# Generated by roxygen2: do not edit by hand

S3method(as.character,iteror)
S3method(as.double,iteror)
S3method(as.list,iteror)
S3method(as.logical,iteror)
S3method(as.numeric,iteror)
S3method(as.vector,iteror)
S3method(concat,default)
S3method(concat,iteror)
S3method(consume,iteror)
S3method(count,default)
S3method(count,iteror)
S3method(format,deque)
S3method(hasNext,ihasNextOr)
S3method(i_enumerate,array)
S3method(i_enumerate,default)
S3method(i_enumerate,iteror)
S3method(ihasNext,default)
S3method(ihasNext,ihasNextOr)
S3method(ihasNext,iteror)
S3method(isplit,data.frame)
S3method(isplit,default)
S3method(iterators::nextElem,iteror)
S3method(iteror,"function")
S3method(iteror,array)
S3method(iteror,connection)
S3method(iteror,data.frame)
S3method(iteror,default)
S3method(iteror,iter)
S3method(iteror,iteror)
S3method(iteror,matrix)
S3method(iteror,python.builtin.object)
S3method(nextOr,iter)
S3method(nextOr,iteror)
S3method(nth,default)
S3method(nth,iteror)
S3method(prod,iteror)
S3method(quantify,default)
S3method(quantify,iteror)
S3method(record,default)
S3method(record,iteror)
S3method(reduce,iteror)
S3method(reticulate::r_to_py,iteror)
S3method(sum,iteror)
S3method(summary,deque)
S3method(take,default)
S3method(take,iteror)
export(concat)
export(consume)
export(count)
export(dotproduct)
export(hasNext)
export(iRNGStream)
export(iRNGSubStream)
export(i_RNGStream)
export(i_RNGSubStream)
export(i_accum)
export(i_apply)
export(i_break)
export(i_chain)
export(i_chunk)
export(i_combinations)
export(i_concat)
export(i_count)
export(i_countn)
export(i_dedup)
export(i_div)
export(i_drop)
export(i_dropwhile)
export(i_enum)
export(i_enumerate)
export(i_keep)
export(i_keepwhile)
export(i_limit)
export(i_map)
export(i_mask)
export(i_pad)
export(i_permutations)
export(i_read.table)
export(i_readBin)
export(i_readLines)
export(i_readdf)
export(i_recycle)
export(i_rep)
export(i_repeat)
export(i_replay)
export(i_rle)
export(i_rleinv)
export(i_roundrobin)
export(i_seq)
export(i_seq_along)
export(i_slice)
export(i_star)
export(i_starmap)
export(i_tabulate)
export(i_tee)
export(i_timeout)
export(i_unique)
export(i_window)
export(i_zip)
export(i_zip_longest)
export(icombinations)
export(icount)
export(icountn)
export(idedup)
export(idiv)
export(ienumerate)
export(igrid)
export(ihasNext)
export(ipermutations)
export(irbinom)
export(iread.table)
export(ireadBin)
export(ireadLines)
export(ireaddf)
export(ireplay)
export(irnbinom)
export(irnorm)
export(irpois)
export(irunif)
export(is.iteror)
export(isample)
export(iseq)
export(iseq_along)
export(isplit)
export(itabulate)
export(iteror)
export(makeIwrapper)
export(nextOr)
export(nth)
export(quantify)
export(record)
export(reduce)
export(take)
importFrom(parallel,nextRNGStream)
importFrom(parallel,nextRNGSubStream)
importFrom(rlang,hash)
importFrom(stats,rbinom)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
