---
title: "Cross reference from iterarors/itertools to iterors"
output:
  html_vignette:
    self_contained: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Cross reference from iterarors/itertools to iterors}
  %\VignetteEncoding{UTF-8}
---

If you are coming from packages iterators/itertools/itertools2, all
functionality should be preserved, but some may be under different
names. Here's a short transition guide:

| iterators/itertools                             | iteror function                                                                                           |   |                                                |                                                         |
|-------------------------------------------------|-----------------------------------------------------------------------------------------------------------|---|------------------------------------------------|---------------------------------------------------------|
| `iterators::iapply(array, margin)`              | [`iteror`](../reference/iteror.html)`(array, by=margin)`                                                  |   |                                                |                                                         |
| `iterators::recycle`                            | [`i_recycle`](../reference/i_recycle.html)                                                                |   |                                                |                                                         |
| `itertools::ifilter(pred, iterable)`            | [`i_keep`](../reference/i_keep.html)`(iterable, pred)`                                                    |   |                                                |                                                         |
| `itertools::ifilterfalse(pred, iterable)`       | [`i_drop`](../reference/i_drop.html)`(iterable, pred)`                                                    |   |                                                |                                                         |
| `itertools::isplitCols(obj, ...)`               | [`iteror.array`](../reference/iteror.array.html)`(obj, by="column", ...)`                                 |   |                                                |                                                         |
| `itertools::isplitIndices(n, ...)`              | [`icount`](../reference/icount.html)`(n, ...)`                                                            |   |                                                |                                                         |
| `itertools::isplitRows(obj, ...)`               | [`iteror.array`](../reference/iteror.html)`(obj, by="row", ...)`                                          |   |                                                |                                                         |
| `itertools::isplitVector(n, )`                  | [`iteror.default`](../reference/iteror.html)`(n, ...)`                                                    |   |                                                |                                                         |
| `itertools::product`                            | [`igrid`](../reference/igrid.html)                                                                        |   |                                                |                                                         |
| `itertools::recycle(iter)`                      | [`i_recycle`](../reference/i_recycle.html)`(iter)`                                                        |   |                                                |                                                         |
| `itertools::recycle(function() {...}, times=N)` | [`iteror`](../reference/iteror.html)`(function() {...}, count=N)`                                         |   |                                                |                                                         |
| `itertools::timeout(secs)`                      | [`i_timeout`](../reference/i_timeout.html)`(secs)`                                                        |   |                                                |                                                         |
| `itertools2::icompress`  | [`i_mask`](../reference/i_mask.html)                                                   |
| `itertools2::icount(start=1, step=3, stop=15)`  | [`iseq`](../reference/iseq.html)`(from=1, to=15, by=3)`                                                   |
| `itertools2::icycle`                            | [`i_recycle`](../reference/i_recycle.html)                                                                |   |                                                |                                                         |
| `itertools2::idropwhile(pred, iterable)`        | [`i_dropwhile`](../reference/i_dropwhile.html)`(iterable, pred)`                                          |   |                                                |                                                         |
| `itertools2::ifilter(pred, iterable)`           | [`i_keep`](../reference/i_keep.html)`(iterable, pred)`                                                    |   |                                                |                                                         |
| `itertools2::ifilterfalse(pred, iterable)`      | [`i_drop`](../reference/i_keep.html)`(iterable, pred)`                                                    |   |                                                |                                                         |
| `itertools2::ilength`                           | [`count`](../reference/count.html)`(obj)`                                                                 |   |                                                |                                                         |
| `itertools2::imap(f, it)`                       | [`i_apply`](../reference/i_apply.html)`(it, f)` or [`i_map`](../reference/i_map.html)`(f, it1, it2, ...)` |   |                                                |                                                         |
| `itertools2::ipairwise(it)`                     | [`i_window`](../reference/i_window.html)`(it, 2)`                                                         |   |                                                |                                                         |
| `itertools2::iproduct`                          | [`igrid`](../reference/igrid.html)                                                                        |   |                                                |                                                         |
| `itertools2::irep_len`                          | [`i_rep`](../reference/i_rep.html)`(length.out=n)`                                                        |   |                                                |                                                         |
| `itertools2::iseq_len`                          | [`icount`](../reference/icount.html)                                                                      |   |                                                |                                                         |
| `itertools2::iseq_along(x)`                     | [`icount`](../reference/icount.html)`(length(x))`                                                         |   |                                                |                                                         |
| `itertools2::itakewhile(pred, iterable)`        | [`i_keepwhile`](../reference/i_keepwhile.html)`(iterable, pred)`                                          |   |                                                |                                                         |
| `itertools2::itripletwise(it)`                  | [`i_window`](../reference/i_window.html)`(it, 3)`                                                         |   |                                                |                                                         |
| `itertools2::iunique_lastseen()`                | [`idedup`](../reference/idedup.html)`()`                                                                |   |                                                |                                                         |

Other notes:

  * `checkFunc` arguments have been removed; instead use [`i_keep`](../reference/i_keep.html)`()`.
